"""
URL configuration for backend project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.contrib import admin
from django.urls import path, include
from rest_framework import routers, serializers, viewsets, filters
from django_filters.rest_framework import DjangoFilterBackend
from django.db.models import Q
from django.utils import timezone
import datetime

from app.models import Order, Waiter, Meal


# Serializers define the API representation.
class WaiterSerializer(serializers.ModelSerializer):
    class Meta:
        model = Waiter
        fields = [
            'id',
            'name',
        ]


# ViewSets define the view behavior.
class WaiterViewSet(viewsets.ModelViewSet):
    queryset = Waiter.objects.all()
    serializer_class = WaiterSerializer


class MealSerializer(serializers.ModelSerializer):
    class Meta:
        model = Meal
        fields = [
            'id',
            'name',
        ]


# ViewSets define the view behavior.
class MealViewSet(viewsets.ModelViewSet):
    queryset = Meal.objects.all()
    serializer_class = MealSerializer
    filter_backends = [filters.OrderingFilter]
    ordering = ['name']


class OrderSerializer(serializers.ModelSerializer):
    waiter_name = serializers.SerializerMethodField()
    status_name = serializers.CharField(source='get_status_display', read_only=True)

    class Meta:
        model = Order
        fields = [
            'id',
            'created_on',
            'updated_on',
            'realization_time',
            'client',
            'waiter',
            'waiter_name',
            'data',
            'status',
            'status_name',
        ]

    def get_waiter_name(self, obj):
        return obj.waiter.name if obj.waiter else ''


# ViewSets define the view behavior.
class OrderViewSet(viewsets.ModelViewSet):
    serializer_class = OrderSerializer
    filter_backends = [DjangoFilterBackend, filters.OrderingFilter]
    filterset_fields = ['waiter']
    ordering = ['-created_on']

    def get_queryset(self):
        five_minutes_ago = timezone.now() + datetime.timedelta(minutes=-15)
        return Order.objects.filter(
            ~Q(status__gte=Order.StatusChoices.READY)
            | Q(updated_on__gte=five_minutes_ago)
        )


# Routers provide an easy way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'meals', MealViewSet, 'meals')
router.register(r'waiters', WaiterViewSet, 'waiters')
router.register(r'orders', OrderViewSet, 'orders')

# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browsable API.
urlpatterns = [
    path('api/', include(router.urls)),
    path('api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    path('admin/', admin.site.urls),
    path('__reload__/', include('django_browser_reload.urls')),
]
